#!/usr/bin/perl -w -I/usr/sausalito/perl -I/usr/sausalito/handlers/base/shell
# $Id: user_shell.pl
#
# toggle shell access on or off for user
#

use CCE;
use Base::User qw(usermod);
use MyShell;
use Base::HomeDir qw(homedir_get_user_dir);
use Unix::PasswdFile;

# Debugging switch:
$DEBUG = "0";
if ($DEBUG) {
    use Sys::Syslog qw( :DEFAULT setlogsock);
    use Data::Dumper;
}

my $cce = new CCE;
$cce->connectfd();

&debug_msg("Available: $MyShell::BAD_SHELL $MyShell::GOOD_SHELL $MyShell::SUSPEND_SHELL $MyShell::LIMITED_SHELL $MyShell::JAIL_SHELL\n");

my $user = $cce->event_object();

my $site_len =  length($user->{site});

# Get the user's standard homedir:
my $alt_root = $user->{volume};
my $homedir = homedir_get_user_dir($user->{name}, $user->{site}, $alt_root);
my ($ok, $shell, $shell_old, $shell_new) = $cce->get($cce->event_oid(), 'Shell');
if (!$ok) {
    $cce->bye('FAIL', '[[base-shell.cantReadUserShell]]');
    exit(1);
}

#&debug_msg(Dumper($shell, $shell_old, $shell_new));

my $jailpasswdfile = '';
my $jaildir = '';
my $shortjaildir = '';
&debug_msg("User $user->{name} homedir: $homedir\n");
if ($user->{capLevels} =~ /&siteAdmin&/) { 
    &debug_msg("User $user->{name} is a siteAdmin\n");
    $jaildir = '/home/.sites/' . $user->{site} . '/./home/users/' . $user->{name};
    $shortjaildir = '/home/.sites/' . $user->{site};
    $jailpasswdfile = '/home/.sites/' . $user->{site} . '/etc/passwd';
}
else {
    &debug_msg("User $user->{name} is NOT a siteAdmin\n");
    $jaildir = '/home/.sites/' . $user->{site} . '/home/./users/' . $user->{name};
    $shortjaildir = '/home/.sites/' . $user->{site} . '/home';
    $jailpasswdfile = '/home/.sites/' . $user->{site} . '/home/etc/passwd';
}
&debug_msg("User $user->{name} potential jaildir: $jaildir - passwd: $jailpasswdfile\n");

# run on create, so make sure we have the info needed
if ($cce->event_is_create() && !scalar(getpwnam($user->{name}))) {
    $cce->bye('DEFER');
    exit(0);
}

# if they have a site defined, verify that site is allowed shell access
# and isn't suspended
my $vsite_shell = {};
if ($user->{site}) {
    my ($site_oid) = $cce->find('Vsite', { 'name' => $user->{site} });
    ($ok, $vsite_shell) = $cce->get($site_oid, 'Shell');

    if (!$ok) {
        $cce->bye('FAIL', '[[base-shell.cceError]]');
        exit(1);
    }

    # only worry if they are trying to toggle from off to on for user
    # shell
    if (($shell_new->{enabled}) && ($vsite_shell->{enabled} eq "0")) {
        $cce->bye('FAIL', '[[base-shell.cantEnableNoVsiteShell]]');
        exit(1);
    }
}

# if the user is suspended or shell is turned off, set shell to non-bash
my $fail_msg = '';
my $new_shell = $MyShell::BAD_SHELL;
my $jail_shell = $MyShell::GOOD_SHELL;
if (($user->{enabled} eq "0") || ($shell->{enabled} eq "0") || ($vsite_shell->{enabled} eq "0")) {
    $fail_msg = '[[base-shell.cantDisableUserShell]]';

    if (!$user->{enabled}) {
        $new_shell = $MyShell::SUSPEND_SHELL;
        $jail_shell = $MyShell::GOOD_SHELL;
        $user->{jailhomedir} = $homedir;
        &debug_msg("Case 1A\n");
    }
    else {
        $new_shell = $MyShell::BAD_SHELL;
        $jail_shell = $MyShell::GOOD_SHELL;
        $user->{jailhomedir} = $homedir;
        &debug_msg("Case 1B\n");
    }
}
elsif ($shell->{enabled} eq "1") {
    $fail_msg = '[[base-shell.cantEnableUserShell]]';
    $new_shell = $MyShell::JAIL_SHELL;
    $jail_shell = $MyShell::LIMITED_SHELL;
    $user->{jailhomedir} = $jaildir;
    &debug_msg("Case 2\n");
}
elsif ($shell->{enabled} eq "2") {
    $fail_msg = '[[base-shell.cantEnableUserShell]]';
    $new_shell = $MyShell::JAIL_SHELL;
    $jail_shell = $MyShell::GOOD_SHELL;
    $user->{jailhomedir} = $jaildir;
    &debug_msg("Case 3\n");
}
elsif ($shell->{enabled} eq "3") {
    $fail_msg = '[[base-shell.cantEnableUserShell]]';
    $new_shell = $MyShell::GOOD_SHELL;
    $jail_shell = $MyShell::GOOD_SHELL;
    $user->{jailhomedir} = $homedir;
    &debug_msg("Case 4\n");
}

# Special case for 'admin' and other special case 'systemAdministrator' Users:
&debug_msg("user->{systemAdministrator}: $user->{systemAdministrator} - user->{site}: $user->{site} - user->{enabled}: $user->{enabled} - shell->{user}: $shell->{user} - shell->{enabled}: $shell->{enabled} - user->{name}: $user->{name}\n");
if (($site_len eq "0") && ($shell->{enabled} eq "1")) {
    $fail_msg = '[[base-shell.cantEnableServerAdminShell]]';
    $new_shell = $MyShell::GOOD_SHELL;
    $jail_shell = $MyShell::GOOD_SHELL;
    $user->{jailhomedir} = $homedir;
    &debug_msg("Case 5 - user->{systemAdministrator: $user->{systemAdministrator} user->{site}: $user->{site} - site_len: $site_len - user->{enabled}: $user->{enabled} - shell->{enabled}: $shell->{enabled} \n");
}

&debug_msg("new shell is $new_shell\n");
my $changeUser = { 'name' => $user->{name}, 'jailhomedir' => $user->{jailhomedir}, 'shell' => $new_shell };
my ($success, $bad_users, $err, $errmsg) = usermod($changeUser);
&debug_msg("Result: $success - $err - $errmsg");

if ($success != 0) {
    &debug_msg("Failed usermod reported in user_shell.pl\n");
    $cce->bye('FAIL', $errmsg);
    exit(1);
}
else {

    # Do not enable Jails for 'systemAdministrator':
    if (($user->{systemAdministrator} eq "1") || ($site_len eq "0")) {
        &debug_msg("User $user->{name} is 'systemAdministrator' || site_len: $site_len - skipping jail modification. #1\n");
    }
    else {
        &debug_msg("User $user->{name} is NOT 'systemAdministrator' - performing jail modification.\n");
        if (($shell->{enabled} eq "1") || ($shell->{enabled} eq "2") || ($shell->{enabled} eq "3")) {

            &debug_msg("Successful usermod, now fixing jails ...\n");
            &debug_msg("Running: /usr/sbin/jk_jailuser -n -j $shortjaildir -s $jail_shell $user->{name}\n");

            # Note: This has a bug and may not update the shell correctly:
            system("/usr/sbin/jk_jailuser -n -j $shortjaildir -s $jail_shell $user->{name}");

            #
            # This drama-queen code is left in for debugging. When a Vsite with jails is created, the service 'jailer'
            # needs to be restarted. Which also creates the $jailpasswdfile for the jail in question. The startup may take a
            # moment, so vsite_shell.pl MUST run the restart of that service with 'nobg' to make sure further CCE processing 
            # is halted until the restart of 'jailer' is done. Otherwise the Unix::PasswdFile attempt below to modify
            # $jailpasswdfile would fail, as the file isn't present yet. Runtime errors are no fun and if this ever pops up
            # again, the below debugging code will help to narrow it down.
            #
            if (! -f $jailpasswdfile) {
                &debug_msg("Jailpasswdfile $jailpasswdfile NOT yet present!\n");
                sleep(60);
                &debug_msg("Wait for $jailpasswdfile completed.\n");
                if (! -f $jailpasswdfile) {
                    &debug_msg("Jailpasswdfile $jailpasswdfile STILL NOT yet present!\n");

                    &debug_msg("Attempting to create Jailpasswdfile $jailpasswdfile\n");
                    my $line_to_write = "root:x:0:0:root:/root:/bin/bash\n";
                    open my $fh, '>>', $jailpasswdfile or die "Could not open $jailpasswdfile: $!";
                    print $fh $line_to_write;
                    close $fh;
                    if (! -f $jailpasswdfile) {
                        &debug_msg("Jailpasswdfile $jailpasswdfile STILL NOT yet present!\n");
                    }
                    else {
                        &debug_msg("Jailpasswdfile $jailpasswdfile is FINALLY present!\n");
                    }
                }
                else {
                    &debug_msg("Jailpasswdfile $jailpasswdfile is FINALLY present!\n");
                }
            }
            else {
                &debug_msg("Jailpasswdfile $jailpasswdfile is present.\n");
            }

            &debug_msg("Unix::PasswdFile: Attempting to modify jailpasswdfile: $jailpasswdfile\n");
            eval {
                $pw = new Unix::PasswdFile $jailpasswdfile;
                $ret = $pw->shell($user->{name}, $jail_shell);
                $pw->commit();
                undef $pw;
            };
            if ($@) {
                # Handle the error
                &debug_msg("Error in Unix::PasswdFile: $@\n");
            }
            else {
                &debug_msg("Return status from Shell mod via Unix::PasswdFile: $ret\n");
            }

            my ($success, $bad_users, $err, $errmsg) = usermod($changeUser);
            &debug_msg("Result from final usermod: $success - $err - $errmsg");

        }
    }
}

$cce->bye('SUCCESS');
exit(0);

#
### Subroutines:
#

sub debug_msg {
    if ($DEBUG) {
        my $msg = shift;
        setlogsock('unix');
        openlog($0,'','user');
        syslog('info', "$ARGV[0]: $msg");
        closelog;
    }
}

# 
# Copyright (c) 2008-2023 Michael Stauber, SOLARSPEED.NET
# Copyright (c) 2008-2023 Team BlueOnyx, BLUEONYX.IT
# Copyright (c) 2003 Sun Microsystems, Inc. 
# All Rights Reserved.
# 
# 1. Redistributions of source code must retain the above copyright 
#    notice, this list of conditions and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright 
#    notice, this list of conditions and the following disclaimer in 
#    the documentation and/or other materials provided with the 
#    distribution.
# 
# 3. Neither the name of the copyright holder nor the names of its 
#    contributors may be used to endorse or promote products derived 
#    from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.
# 
# You acknowledge that this software is not designed or intended for 
# use in the design, construction, operation or maintenance of any 
# nuclear facility.
# 