#!/usr/bin/perl -I/usr/sausalito/perl -I/usr/sausalito/handlers/base/shell
# $Id: vsite_gauth.pl
#
# Toggle GoogleAuthentication on or off for Users when the feature is enabled/disabled for the Vsite
#

use CCE;

# Debugging switch:
$DEBUG = "0";
if ($DEBUG) {
    use Sys::Syslog qw( :DEFAULT setlogsock);
    use Data::Dumper;
}

my $cce = new CCE;
$cce->connectfd();

my $Vsite_Obj = $cce->event_object();
my ($ok, $Vsite_Shell_Object) = $cce->get($Vsite_Obj->{'OID'}, "Shell");
if (!$ok) {
    $cce->bye('FAIL', '[[base-shell.cantReadVsiteShell]]');
    exit(1);
}

# Get 'System' . 'SSH':
my ($sys_oid) = $cce->find('System');
($ok, $System_SSH_Object) = $cce->get($sys_oid, "SSH");

# Sense check:
if (($System_SSH_Object->{'GoogleAuthentication'} eq '0') && ($Vsite_Shell_Object->{'GoogleAuthentication'} eq '1')) {
    # Override: System says 'GoogleAuthentication' is off for the server. So it cannot be on for the Vsite:
    $Vsite_Shell_Object->{'GoogleAuthentication'} = '0';
}

&debug_msg("GoogleAuthentication is set to $Vsite_Shell_Object->{'GoogleAuthentication'} for Vsite $Vsite_Obj->{'fqdn'}\n");

# Find all Users of this Vsite:
@vsite_user_oids = ();
(@vsite_user_oids) = $cce->findx('User', { 'site' => $Vsite_Obj->{name} });

for $user_oid (@vsite_user_oids) {
    ($ok, $user_SSH) = $cce->get($user_oid, 'SSH');

    if (($user_SSH->{'GoogleAuthentication'} eq '1') && ($Vsite_Shell_Object->{'GoogleAuthentication'} eq '0')) {
        # Vsite has 'GoogleAuthentication' OFF. Turn it OFF for all Users:
        ($ok) = $cce->set($user_oid, 'SSH', { 'GoogleAuthentication' => '0' });
    }
    if (($user_SSH->{'GoogleAuthentication'} eq '0') && ($Vsite_Shell_Object->{'GoogleAuthentication'} eq '1')) {
        # Vsite has 'GoogleAuthentication' ON. Turn it ON for all Users:
        ($ok) = $cce->set($user_oid, 'SSH', { 'GoogleAuthentication' => '1' });
    }
}

$cce->bye('SUCCESS');
exit(0);

#
### Subroutines:
#

sub debug_msg {
    if ($DEBUG) {
        my $msg = shift;
        setlogsock('unix');
        openlog($0,'','user');
        syslog('info', "$ARGV[0]: $msg");
        closelog;
    }
}

# 
# Copyright (c) 2008-2023 Michael Stauber, SOLARSPEED.NET
# Copyright (c) 2008-2023 Team BlueOnyx, BLUEONYX.IT
# All Rights Reserved.
# 
# 1. Redistributions of source code must retain the above copyright 
#    notice, this list of conditions and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright 
#    notice, this list of conditions and the following disclaimer in 
#    the documentation and/or other materials provided with the 
#    distribution.
# 
# 3. Neither the name of the copyright holder nor the names of its 
#    contributors may be used to endorse or promote products derived 
#    from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.
# 
# You acknowledge that this software is not designed or intended for 
# use in the design, construction, operation or maintenance of any 
# nuclear facility.
# 
